// fancyprint_test.cpp
// Glenn G. Chappell
// 19 Oct 2013
//
// For CS 202 Fall 2013
// Test program for class FancyPrint & derived classes
// Used in Assignment 5, Exercise A

// Includes for code to be tested
#include "fancyprint.h" // For class FancyPrint
#include "fancyprint.h" // Double inclusion test
#include "starprint.h"  // For class StarPrint
#include "starprint.h"  // Double inclusion test
#include "lineprint.h"  // For class LinePrint
#include "lineprint.h"  // Double inclusion test
#include "backprint.h"  // For class BackPrint
#include "backprint.h"  // Double inclusion test

// Includes for testing package & code common to all test programs
#include <iostream>     // for std::cout, std::endl, std::cin
#include <string>       // for std::string
#include <stdexcept>    // for std::runtime_error

// Additional includes for this test program
#include <sstream>      // for std::ostringstream


// ************************************************************************
// Testing Package:
//     Class Tester - For Tracking Tests
// ************************************************************************


// class Tester
// For extremely simple unit testing.
// Keeps track of number of tests and number of passes.
// Use test (with success/failure parameter) to do a test.
// Get results with numTests, numPassed, numFailed, allPassed.
// Restart testing with reset.
// Invariants:
//     countTests_ == number of tests (calls to test) since last reset.
//     countPasses_ == number of times function test called with true param
//      since last reset.
//     0 <= countPasses_ <= countTests_.
//     tolerance_ >= 0.
class Tester {

// ***** Tester: ctors, dctor, op= *****
public:

    // Default ctor
    // Sets countTests_, countPasses_ to zero, tolerance_ to given value
    // Pre: None.
    // Post:
    //     numTests == 0, countPasses == 0, tolerance_ == abs(theTolerance)
    // Does not throw (No-Throw Guarantee)
    Tester(double theTolerance = 0.0000001)
        :countTests_(0),
         countPasses_(0),
         tolerance_(theTolerance >= 0 ? theTolerance : -theTolerance)
    {}

    // Compiler-generated copy ctor, copy op=, dctor are used

// ***** Tester: general public functions *****
public:

    // test
    // Handles single test, param indicates pass/fail
    // Pre: None.
    // Post:
    //     countTests_ incremented
    //     countPasses_ incremented if (success)
    //     Message indicating test name (if given)
    //      and pass/fail printed to cout
    // Does not throw (No-Throw Guarantee)
    //  - Assuming exceptions have not been turned on for cout.
    void test(bool success,
              const std::string & testName = "")
    {
        ++countTests_;
        if (success) ++countPasses_;

        std::cout << "    ";
        if (testName != "")
        {
            std::cout << "Test: "
                      << testName
                      << " - ";
        }
        std::cout << (success ? "passed" : "********** FAILED **********")
                  << std::endl;
    }

    // ftest
    // Does single floating-point test.
    // Tests passes iff difference of first two values is <= tolerance.
    // Pre: None.
    // Post:
    //     countTests_ incremented
    //     countPasses_ incremented if (abs(val1-val2) <= tolerance_)
    //     Message indicating test name (if given)
    //      and pass/fail printed to cout
    // Does not throw (No-Throw Guarantee)
    void ftest(double val1,
               double val2,
               const std::string & testName = "")
    { test(val1-val2 <= tolerance_ && val2-val1 <= tolerance_, testName); }

    // reset
    // Resets *this to default constructed state
    // Pre: None.
    // Post:
    //     countTests_ == 0, countPasses_ == 0
    // Does not throw (No-Throw Guarantee)
    void reset()
    {
        countTests_ = 0;
        countPasses_ = 0;
    }

    // numTests
    // Returns the number of tests that have been done since last reset 
    // Pre: None.
    // Post:
    //     return == countTests_
    // Does not throw (No-Throw Guarantee)
    int numTests() const
    { return countTests_; }

    // numPassed
    // Returns the number of tests that have passed since last reset
    // Pre: None.
    // Post:
    //     return == countPasses_
    // Does not throw (No-Throw Guarantee)
    int numPassed() const
    { return countPasses_; }

    // numFailed
    // Returns the number of tests that have not passed since last reset
    // Pre: None.
    // Post:
    //     return + countPasses_ == numTests_
    // Does not throw (No-Throw Guarantee)
    int numFailed() const
    { return countTests_ - countPasses_; }

    // allPassed
    // Returns true if all tests since last reset have passed
    // Pre: None.
    // Post:
    //     return == (countPasses_ == countTests_)
    // Does not throw (No-Throw Guarantee)
    bool allPassed() const
    { return countPasses_ == countTests_; }

    // setTolerance
    // Sets tolerance_ to given value
    // Pre: None.
    // Post:
    //     tolerance_ = abs(theTolerance)
    // Does not throw (No-Throw Guarantee)
    void setTolerance(double theTolerance)
    { tolerance_ = (theTolerance >= 0 ? theTolerance : -theTolerance); }

// ***** Tester: data members *****
private:

    int countTests_;    // Number of tests done since last reset
    int countPasses_;   // Number of tests passed since last reset
    double tolerance_;  // Tolerance for floating-point near-equality tests

};  // end class Tester


// ************************************************************************
// Testing Package:
//     Class TypeCheck - Helper Class for Type Checking
// ************************************************************************


// class TypeCheck
// This class exists in order to have static member function check, which
// takes a parameter of a given type, by reference. Objects of type
// TypeCheck<T> cannot be created.
// Usage:
//     TypeCheck<MyType>::check(x)
//     returns true if the type of x is (MyType) or (const MyType),
//     otherwise false.
// Invariants: None.
// Requirements on Types: None.
template<typename T>
class TypeCheck {

private:

    // Uncopyable class. Do not define copy ctor, copy assn.
    TypeCheck(const TypeCheck &);
    TypeCheck<T> & operator=(const TypeCheck &);

    // Compiler-generated dctor is used (but irrelevant).

public:

    // check
    // The function and function template below simulate a single function
    // that takes a single parameter, and returns true iff the parameter has
    // type T or (const T).

    // check (reference-to-const T)
    // Pre: None.
    // Post:
    //     Return is true.
    // Does not throw (No-Throw Guarantee)
    static bool check(const T & param)
    { return true; }

    // check (reference-to-const non-T)
    // Pre: None.
    // Post:
    //     Return is false.
    // Requirements on types: None.
    // Does not throw (No-Throw Guarantee)
    template <typename OtherType>
    static bool check(const OtherType & param)
    { return false; }

};  // End class TypeCheck


// ************************************************************************
// Testing Package:
//     Class Counter - Helper Class for Counting Calls & Objects, Throwing
// ************************************************************************


// class Counter
// Item type for counting ctor, dctor, op= calls, counting existing
//  objects, and possibly throwing on copy. Has operator< (which always
//  returns false), allowing it to be the value type of a sorted container.
// If static member copyThrow_ is set, then copy ctor and copy assn throw
//  std::runtime_error. Exception object's "what" member is set to "C" by
//  the copy ctor and "A" by copy assn.
// Increments static data member ctorCount_ on default construction and
//  successful copy construction. Increments static data member assnCount_
//  on successful copy assignment. Increments static data member
//  dctorCount_ on destruction.
// Increments static data member existing_ on construction, and decrements
//  it on destruction.
// Static data member maxExisting_ is highest value of existing_ since last
//  reset, or start of program if reset has never been called.
// Invariants:
//     Counter::existing_ is number of existing objects of this class.
//     Counter::ctorCount_ is number of successful ctor calls since
//      most recent call to reset, or start of program if reset has never
//      been called.
//     Counter::dctorCount_ is (similarly) number of dctor calls.
//     Counter::assnCount_ is (similarly) number of copy assn calls.
//     Counter::maxExisting_ is (similarly) highest value existing_ has
//      assumed.
class Counter {

// ***** Counter: Ctors, dctor, op= *****
public:

    // Default ctor
    // Pre: None.
    // Post:
    //     (ctorCount_ has been incremented.)
    //     (existing_ has been incremented.)
    // Does not throw (No-Throw Guarantee)
    Counter()
    {
        ++existing_;
        if (existing_ > maxExisting_)
            maxExisting_ = existing_;
        ++ctorCount_;
    }

    // Copy ctor
    // Throws std::runtime_error if copyThrow_.
    // Pre: None.
    // Post:
    //     (ctorCount_ has been incremented.)
    //     (existing_ has been incremented.)
    // May throw std::runtime_error
    // Strong Guarantee
    Counter(const Counter & other)
    {
        if (copyThrow_)
            throw std::runtime_error("C");
        ++existing_;
        if (existing_ > maxExisting_)
            maxExisting_ = existing_;
        ++ctorCount_;
    }

    // Copy assignment
    // Throws std::runtime_error if copyThrow_.
    // Pre: None.
    // Post:
    //     Return value is *this.
    //     (assnCount_ has been incremented.)
    // May throw std::runtime_error
    // Strong Guarantee
    Counter & operator=(const Counter & rhs)
    {
        if (copyThrow_)
            throw std::runtime_error("A");
        ++assnCount_;
        return *this;
    }

    // Dctor
    // Pre: None.
    // Post:
    //     (dctorCount_ has been incremented.)
    //     (existing_ has been decremented.)
    // Does not throw (No-Throw Guarantee)
    ~Counter()
    {
        --existing_;
        ++dctorCount_;
    }

// ***** Counter: Functions dealing with count *****
public:

    // reset
    // Pre: None.
    // Post:
    //     maxExisting_ == existing_.
    //     ctorCount_ == 0.
    //     dctorCount_ == 0.
    //     assnCount_ == 0.
    //     copyThrow_ == shouldThrow.
    // Does not throw (No-Throw Guarantee)
    static void reset(bool shouldThrow = false)
    {
        maxExisting_ = existing_;
        ctorCount_ = 0;
        dctorCount_ = 0;
        assnCount_ = 0;
        copyThrow_ = shouldThrow;
    }

    // getExisting
    // Pre: None.
    // Post:
    //     return == existing_.
    // Does not throw (No-Throw Guarantee)
    static int getExisting()
    { return existing_; }

    // getMaxExisting
    // Pre: None.
    // Post:
    //     return == maxExisting_.
    // Does not throw (No-Throw Guarantee)
    static int getMaxExisting()
    { return maxExisting_; }

    // getCtorCount
    // Pre: None.
    // Post:
    //     return == ctorCount_.
    // Does not throw (No-Throw Guarantee)
    static int getCtorCount()
    { return ctorCount_; }

    // getDctorCount
    // Pre: None.
    // Post:
    //     return == dctorCount_.
    // Does not throw (No-Throw Guarantee)
    static int getDctorCount()
    { return dctorCount_; }

    // getAssnCount
    // Pre: None.
    // Post:
    //     return == assnCount_.
    // Does not throw (No-Throw Guarantee)
    static int getAssnCount()
    { return assnCount_; }

    // setCopyThrow
    // Pre: None.
    // Post:
    //     copyThrow_ == shouldThrow
    // Does not throw (No-Throw Guarantee)
    static void setCopyThrow(bool shouldThrow)
    { copyThrow_ = shouldThrow; }

// ***** Counter: Data Members *****
private:

    static int existing_;     // # of existing objects
    static int maxExisting_;  // Max # of existing objects
    static int ctorCount_;    // # of successful (non-throwing) ctor calls
    static int dctorCount_;   // # of dctor calls
    static int assnCount_;    // # of successful (non-throwing) copy = calls
    static bool copyThrow_;   // true if copy operations (ctor, =) throw

};  // End class Counter

// Definition of static data member of class Counter
int Counter::existing_ = 0;
int Counter::maxExisting_ = 0;
int Counter::ctorCount_ = 0;
int Counter::dctorCount_ = 0;
int Counter::assnCount_ = 0;
bool Counter::copyThrow_ = false;


// operator< (Counter)
// Dummy-ish operator<, forming a strict weak order for Counter class
// Returns false (which is legal for a strict weak order; all objects of
//  type Counter are equivalent).
// Pre: None.
// Post:
//     Return value == false.
// Does not throw (No-Throw Guarantee)
bool operator<(const Counter & a,
               const Counter & b)
{ return false; }


// ************************************************************************
// Definitions for this test suite
// ************************************************************************


// (NONE)


// ************************************************************************
// Test Suite Functions
// ************************************************************************


// test_FancyPrint_StarPrint
// Test suite for class FancyPrint: derived class StarPrint
// Pre: None.
// Post:
//     Pass/fail status of tests have been registered with t.
//     Appropriate messages have been printed to cout.
// Does not throw (No-Throw Guarantee)
void test_FancyPrint_StarPrint(Tester & t)
{
    std::cout << "Test Suite: FancyPrint derived class StarPrint"
              << std::endl;

    string m;  // Holds message values
    int c;     // Holds count values

    {  // Default constructed
    const StarPrint x;
    m = x.getMessage();
    const FancyPrint * p = &x;
    t.test(m == "",
      "StarPrint - default ctor - msg");
    c = x.getCount();
    t.test(c == 1,
      "StarPrint - default ctor - count");
    std::ostringstream os1;
    x.print(os1);
    t.test(os1.str() == "*  *\n",
      "StarPrint - default ctor - print");
    //m = p->getMessage();
    //t.test(m == "",
    //  "StarPrint - default ctor - msg via base ptr");
    //c = p->getCount();
    //t.test(c == 1,
    //  "StarPrint - default ctor - count via base ptr");
    std::ostringstream os2;
    p->print(os2);
    t.test(os2.str() == "*  *\n",
      "StarPrint - default ctor - print via base ptr");
    }

    {  // Constructed from string
    const string s = "bludgeon";
    const StarPrint x(s);
    const FancyPrint * p = &x;
    m = x.getMessage();
    t.test(m == "bludgeon",
      "StarPrint - construct from str - msg");
    c = x.getCount();
    t.test(c == 1,
      "StarPrint - construct from str - count");
    std::ostringstream os1;
    x.print(os1);
    t.test(os1.str() == "* bludgeon *\n",
      "StarPrint - construct from str - print");
    //m = p->getMessage();
    //t.test(m == "bludgeon",
    //  "StarPrint - construct from str - msg via base ptr");
    //c = p->getCount();
    //t.test(c == 1,
    //  "StarPrint - construct from str - count via base ptr");
    std::ostringstream os2;
    p->print(os2);
    t.test(os2.str() == "* bludgeon *\n",
      "StarPrint - construct from str - print via base ptr");
    }

    {  // Constructed from string,int
    const string s = "atrophy";
    const StarPrint x(s, 2);
    const FancyPrint * p = &x;
    m = x.getMessage();
    t.test(m == "atrophy",
      "StarPrint - construct from str,int - msg");
    c = x.getCount();
    t.test(c == 2,
      "StarPrint - construct from str,int - count");
    std::ostringstream os1;
    x.print(os1);
    t.test(os1.str() == "** atrophy **\n",
      "StarPrint - construct from str,int - print");
    //m = p->getMessage();
    //t.test(m == "atrophy",
    //  "StarPrint - construct from str,int - msg via base ptr");
    //c = p->getCount();
    //t.test(c == 2,
    //  "StarPrint - construct from str,int - count via base ptr");
    std::ostringstream os2;
    p->print(os2);
    t.test(os2.str() == "** atrophy **\n",
      "StarPrint - construct from str,int - print via base ptr");
    }

    {  // Use set-functions
    const string s = "xy";
    const int i = 4;
    StarPrint x;
    x.setMessage(s);
    x.setCount(i);
    const FancyPrint * p = &x;
    m = x.getMessage();
    t.test(m == "xy",
      "StarPrint - use set-functions - msg");
    c = x.getCount();
    t.test(c == 4,
      "StarPrint - use set-functions - count");
    std::ostringstream os1;
    x.print(os1);
    t.test(os1.str() == "**** xy ****\n",
      "StarPrint - use set-functions - print");
    //m = p->getMessage();
    //t.test(m == "xy",
    //  "StarPrint - use set-functions - msg via base ptr");
    //c = p->getCount();
    //t.test(c == 4,
    //  "StarPrint - use set-functions - count via base ptr");
    std::ostringstream os2;
    p->print(os2);
    t.test(os2.str() == "**** xy ****\n",
      "StarPrint - use set-functions - print via base ptr");
    }

    /*
    {  // Set via base ptr
    const string s = "$&(((";
    const int i = 3;
    StarPrint x;
    FancyPrint * p = &x;
    p->setMessage(s);
    p->setCount(i);
    m = x.getMessage();
    t.test(m == "$&(((",
      "StarPrint - set via base ptr - msg");
    c = x.getCount();
    t.test(c == 3,
      "StarPrint - set via base ptr - count");
    std::ostringstream os1;
    x.print(os1);
    t.test(os1.str() == "*** $&((( ***\n",
      "StarPrint - set via base ptr - print");
    //m = p->getMessage();
    //t.test(m == "$&(((",
    //  "StarPrint - set via base ptr - msg via base ptr");
    //c = p->getCount();
    //t.test(c == 3,
    //  "StarPrint - set via base ptr - count via base ptr");
    std::ostringstream os2;
    p->print(os2);
    t.test(os2.str() == "*** $&((( ***\n",
      "StarPrint - set via base ptr - print via base ptr");
    }
    */
}


// test_FancyPrint_LinePrint
// Test suite for class FancyPrint: derived class LinePrint
// Pre: None.
// Post:
//     Pass/fail status of tests have been registered with t.
//     Appropriate messages have been printed to cout.
// Does not throw (No-Throw Guarantee)
void test_FancyPrint_LinePrint(Tester & t)
{
    std::cout << "Test Suite: FancyPrint derived class LinePrint"
              << std::endl;

    string m;  // Holds message values
    int c;     // Holds count values

    {  // Default constructed
    const LinePrint x;
    m = x.getMessage();
    const FancyPrint * p = &x;
    t.test(m == "",
      "LinePrint - default ctor - msg");
    c = x.getCount();
    t.test(c == 1,
      "LinePrint - default ctor - count");
    std::ostringstream os1;
    x.print(os1);
    t.test(os1.str() == "\n\n\n",
      "LinePrint - default ctor - print");
    //m = p->getMessage();
    //t.test(m == "",
    //  "LinePrint - default ctor - msg via base ptr");
    //c = p->getCount();
    //t.test(c == 1,
    //  "LinePrint - default ctor - count via base ptr");
    std::ostringstream os2;
    p->print(os2);
    t.test(os2.str() == "\n\n\n",
      "LinePrint - default ctor - print via base ptr");
    }

    {  // Constructed from string
    const string s = "bludgeon";
    const LinePrint x(s);
    const FancyPrint * p = &x;
    m = x.getMessage();
    t.test(m == "bludgeon",
      "LinePrint - construct from str - msg");
    c = x.getCount();
    t.test(c == 1,
      "LinePrint - construct from str - count");
    std::ostringstream os1;
    x.print(os1);
    t.test(os1.str() == "--------\nbludgeon\n--------\n",
      "LinePrint - construct from str - print");
    //m = p->getMessage();
    //t.test(m == "bludgeon",
    //  "LinePrint - construct from str - msg via base ptr");
    //c = p->getCount();
    //t.test(c == 1,
    //  "LinePrint - construct from str - count via base ptr");
    std::ostringstream os2;
    p->print(os2);
    t.test(os2.str() == "--------\nbludgeon\n--------\n",
      "LinePrint - construct from str - print via base ptr");
    }

    {  // Constructed from string,int
    const string s = "atrophy";
    const LinePrint x(s, 2);
    const FancyPrint * p = &x;
    m = x.getMessage();
    t.test(m == "atrophy",
      "LinePrint - construct from str,int - msg");
    c = x.getCount();
    t.test(c == 2,
      "LinePrint - construct from str,int - count");
    std::ostringstream os1;
    x.print(os1);
    t.test(os1.str() == "-------\n-------\natrophy\n-------\n-------\n",
      "LinePrint - construct from str,int - print");
    //m = p->getMessage();
    //t.test(m == "atrophy",
    //  "LinePrint - construct from str,int - msg via base ptr");
    //c = p->getCount();
    //t.test(c == 2,
    //  "LinePrint - construct from str,int - count via base ptr");
    std::ostringstream os2;
    p->print(os2);
    t.test(os2.str() == "-------\n-------\natrophy\n-------\n-------\n",
      "LinePrint - construct from str,int - print via base ptr");
    }

    {  // Use set-functions
    const string s = "xy";
    const int i = 4;
    LinePrint x;
    x.setMessage(s);
    x.setCount(i);
    const FancyPrint * p = &x;
    m = x.getMessage();
    t.test(m == "xy",
      "LinePrint - use set-functions - msg");
    c = x.getCount();
    t.test(c == 4,
      "LinePrint - use set-functions - count");
    std::ostringstream os1;
    x.print(os1);
    t.test(os1.str() == "--\n--\n--\n--\nxy\n--\n--\n--\n--\n",
      "LinePrint - use set-functions - print");
    //m = p->getMessage();
    //t.test(m == "xy",
    //  "LinePrint - use set-functions - msg via base ptr");
    //c = p->getCount();
    //t.test(c == 4,
    //  "LinePrint - use set-functions - count via base ptr");
    std::ostringstream os2;
    p->print(os2);
    t.test(os2.str() == "--\n--\n--\n--\nxy\n--\n--\n--\n--\n",
      "LinePrint - use set-functions - print via base ptr");
    }

    /*
    {  // Set via base ptr
    const string s = "$&(((";
    const int i = 3;
    LinePrint x;
    FancyPrint * p = &x;
    p->setMessage(s);
    p->setCount(i);
    m = x.getMessage();
    t.test(m == "$&(((",
      "LinePrint - set via base ptr - msg");
    c = x.getCount();
    t.test(c == 3,
      "LinePrint - set via base ptr - count");
    std::ostringstream os1;
    x.print(os1);
    t.test(os1.str() == "-----\n-----\n-----\n$&(((\n-----\n-----\n-----\n",
      "LinePrint - set via base ptr - print");
    //m = p->getMessage();
    //t.test(m == "$&(((",
    //  "LinePrint - set via base ptr - msg via base ptr");
    //c = p->getCount();
    //t.test(c == 3,
    //  "LinePrint - set via base ptr - count via base ptr");
    std::ostringstream os2;
    p->print(os2);
    t.test(os2.str() == "-----\n-----\n-----\n$&(((\n-----\n-----\n-----\n",
      "LinePrint - set via base ptr - print via base ptr");
    }
    */
}


// test_FancyPrint_BackPrint
// Test suite for class FancyPrint: derived class BackPrint
// Pre: None.
// Post:
//     Pass/fail status of tests have been registered with t.
//     Appropriate messages have been printed to cout.
// Does not throw (No-Throw Guarantee)
void test_FancyPrint_BackPrint(Tester & t)
{
    std::cout << "Test Suite: FancyPrint derived class BackPrint"
              << std::endl;

    string m;  // Holds message values
    int c;     // Holds count values

    {  // Default constructed
    const BackPrint x;
    m = x.getMessage();
    const FancyPrint * p = &x;
    t.test(m == "",
      "BackPrint - default ctor - msg");
    c = x.getCount();
    t.test(c == 1,
      "BackPrint - default ctor - count");
    std::ostringstream os1;
    x.print(os1);
    t.test(os1.str() == "\n",
      "BackPrint - default ctor - print");
    //m = p->getMessage();
    //t.test(m == "",
    //  "BackPrint - default ctor - msg via base ptr");
    //c = p->getCount();
    //t.test(c == 1,
    //  "BackPrint - default ctor - count via base ptr");
    std::ostringstream os2;
    p->print(os2);
    t.test(os2.str() == "\n",
      "BackPrint - default ctor - print via base ptr");
    }

    {  // Constructed from string
    const string s = "bludgeon";
    const BackPrint x(s);
    const FancyPrint * p = &x;
    m = x.getMessage();
    t.test(m == "bludgeon",
      "BackPrint - construct from str - msg");
    c = x.getCount();
    t.test(c == 1,
      "BackPrint - construct from str - count");
    std::ostringstream os1;
    x.print(os1);
    t.test(os1.str() == "noegdulb\n",
      "BackPrint - construct from str - print");
    //m = p->getMessage();
    //t.test(m == "bludgeon",
    //  "BackPrint - construct from str - msg via base ptr");
    //c = p->getCount();
    //t.test(c == 1,
    //  "BackPrint - construct from str - count via base ptr");
    std::ostringstream os2;
    p->print(os2);
    t.test(os2.str() == "noegdulb\n",
      "BackPrint - construct from str - print via base ptr");
    }

    {  // Constructed from string,int
    const string s = "atrophy";
    const BackPrint x(s, 2);
    const FancyPrint * p = &x;
    m = x.getMessage();
    t.test(m == "atrophy",
      "BackPrint - construct from str,int - msg");
    c = x.getCount();
    t.test(c == 2,
      "BackPrint - construct from str,int - count");
    std::ostringstream os1;
    x.print(os1);
    t.test(os1.str() == "yhporta\nyhporta\n",
      "BackPrint - construct from str,int - print");
    //m = p->getMessage();
    //t.test(m == "atrophy",
    //  "BackPrint - construct from str,int - msg via base ptr");
    //c = p->getCount();
    //t.test(c == 2,
    //  "BackPrint - construct from str,int - count via base ptr");
    std::ostringstream os2;
    p->print(os2);
    t.test(os2.str() == "yhporta\nyhporta\n",
      "BackPrint - construct from str,int - print via base ptr");
    }

    {  // Use set-functions
    const string s = "xy";
    const int i = 4;
    BackPrint x;
    x.setMessage(s);
    x.setCount(i);
    const FancyPrint * p = &x;
    m = x.getMessage();
    t.test(m == "xy",
      "BackPrint - use set-functions - msg");
    c = x.getCount();
    t.test(c == 4,
      "BackPrint - use set-functions - count");
    std::ostringstream os1;
    x.print(os1);
    t.test(os1.str() == "yx\nyx\nyx\nyx\n",
      "BackPrint - use set-functions - print");
    //m = p->getMessage();
    //t.test(m == "xy",
    //  "BackPrint - use set-functions - msg via base ptr");
    //c = p->getCount();
    //t.test(c == 4,
    //  "BackPrint - use set-functions - count via base ptr");
    std::ostringstream os2;
    p->print(os2);
    t.test(os2.str() == "yx\nyx\nyx\nyx\n",
      "BackPrint - use set-functions - print via base ptr");
    }

    /*
    {  // Set via base ptr
    const string s = "$&(((";
    const int i = 3;
    BackPrint x;
    FancyPrint * p = &x;
    p->setMessage(s);
    p->setCount(i);
    m = x.getMessage();
    t.test(m == "$&(((",
      "BackPrint - set via base ptr - msg");
    c = x.getCount();
    t.test(c == 3,
      "BackPrint - set via base ptr - count");
    std::ostringstream os1;
    x.print(os1);
    t.test(os1.str() == "(((&$\n(((&$\n(((&$\n",
      "BackPrint - set via base ptr - print");
    //m = p->getMessage();
    //t.test(m == "$&(((",
    //  "BackPrint - set via base ptr - msg via base ptr");
    //c = p->getCount();
    //t.test(c == 3,
    //  "BackPrint - set via base ptr - count via base ptr");
    std::ostringstream os2;
    p->print(os2);
    t.test(os2.str() == "(((&$\n(((&$\n(((&$\n",
      "BackPrint - set via base ptr - print via base ptr");
    }
    */
}


// test_FancyPrint
// Test suite for class FancyPrint
// Uses other test-suite functions
// Pre: None.
// Post:
//     Pass/fail status of tests have been registered with t.
//     Appropriate messages have been printed to cout.
// Does not throw (No-Throw Guarantee)
void test_FancyPrint(Tester & t)
{
    // Do all the test suites
    std::cout << "TEST SUITES FOR CLASS FancyPrint" << std::endl;
    test_FancyPrint_StarPrint(t);
    test_FancyPrint_LinePrint(t);
    test_FancyPrint_BackPrint(t);
}


// ************************************************************************
// Main program
// ************************************************************************


// main
// Runs class FancyPrint test suite, prints results to cout.
int main()
{
    Tester t;
    test_FancyPrint(t);

    std::cout << std::endl;
    if (t.allPassed())
    {
        std::cout << "All tests successful" 
                  << std::endl;
    }
    else
    {
        std::cout << "Tests ********** UNSUCCESSFUL **********"
                  << std::endl;
    }
    std::cout << std::endl;

    std::cout << "Press ENTER to quit ";
    while (std::cin.get() != '\n') ;

    return 0;
}

